import os
import sys
import logging
import argparse

from COMPS import Client
from COMPS.Data import Experiment, Simulation

logger = logging.getLogger(__name__)

##########################

utility_metadata = {
    'aliases': [ 'getexpout' ],
    'help': 'Download output files from each simulation in an experiment',
    'description': '''This utility downloads one or more output files from each simulation in an experiment, writing them relative to the current directory, in the hierarchy:
    ./<exp-id>/<sim-id>/<filename>
''',
    'epilog': '''examples:
  %(prog)s 11111111-2222-3333-4444-000000000000 insetchart.json
  %(prog)s 11111111-2222-3333-4444-000000000000 stdout.txt,stderr.txt
'''
}

def fill_parser(p):
    p.add_argument('experiment_id', help='Id of the experiment containing the simulations to download files from')
    p.add_argument('filename', help='Name(s) of the file(s) to download from each simulation.  This can be a comma-delimited list, or an actual (python) list if calling from code')
    p.add_argument('--overwrite', '-ow', action='store_true', help='Overwrite local files if they already exist (default is to skip if a local file with the same subdir/name exists)')
    p.add_argument('--casesensitive', '-cs', action='store_true', help='Make filename comparisons case-sensitive (default is case-insensitive, i.e. ignoring case)')

##########################

def get_files( experiment_id, files_to_get, overwrite=False, casesensitive=False ):
    if not isinstance(files_to_get, list):
        files_to_get = [ files_to_get ]

    files_to_get_int = files_to_get if casesensitive else [ f.lower() for f in files_to_get ]

    exp = Experiment.get(experiment_id)
    logger.info(f'Found experiment {exp.id}')

    try:
        os.mkdir(str(exp.id))
    except FileExistsError:
        pass

    sims = exp.get_simulations()
    logger.info(f'{len(sims)} child simulations found')

    missing_files = False
    hit_fileexists = False

    for sim in sims:
        so = sim.retrieve_output_file_info(None)
        logger.debug(f'sim {sim.id} - {len(so)} output files found')

        simpath = os.path.join(str(exp.id), str(sim.id))

        try:
            os.mkdir(simpath)
        except FileExistsError:
            pass

        found_file_num = 0

        for files in so:
            fn_comp = files.friendly_name if casesensitive else files.friendly_name.lower()

            if fn_comp in files_to_get_int:
                found_file_num += 1
                filepath = os.path.join(simpath, files.friendly_name)
                logger.info(filepath)
                oba = sim.retrieve_output_files_from_info([files])
                try:
                    with open(filepath, 'wb' if overwrite else 'xb') as outfile:
                        outfile.write(oba[0])
                except FileExistsError as e:
                    logger.error(f'Output file already exists at {filepath}.  Skipping...')
                    logger.debug(e, exc_info=True)
                    hit_fileexists = True

        # if we didn't find enough matching files, spit out a message
        if found_file_num < len(files_to_get_int):
            logger.warn(f'Didn\'t find files to match all requested for sim {sim.id}')
            missing_files = True

    if missing_files:
        logger.warn('')
        logger.warn(f'Couldn\'t find files matching requested for some sims (possible typo, casing issue, or requested an input-asset?)')

    if hit_fileexists:
        logger.warn('')
        logger.warn('Skipped downloading of some files because they already exist locally.  Rerun using \'--overwrite\' if you want these overwritten instead')


def main(args):
    Client.login(args.comps_server)
    get_files(args.experiment_id, args.filename.split(','), args.overwrite, args.casesensitive)
